package net.grandnature.android.soundball;

import net.grandnature.android.soundball.sound.Note;
import net.grandnature.android.soundball.sound.SoundballPlayer;
import android.content.Context;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;

/**
 * ボール
 */
public class Soundball extends SoundballCharacter {
	private SoundballPlayer player; // プレイヤー
	private Note note;// 音
	private int diagonal; // 対角線の量
	private boolean holding; // 掴んでいるかどうかを判別するフラグ

	public Soundball(Context context, Drawable drawable, Rect viewRect) {
		super(context, drawable, viewRect);
	}
	
	@Override
	protected void init() {
		player = new SoundballPlayer();
		note = new Note(8000);
		diagonal = (int)Math.sqrt(Math.pow(viewRect.height(), 2) + Math.pow(viewRect.width(),2));
		player.play(note);
	}


	/**
	 * 掴んでいるかどうかを設定する
	 * @param holding　掴んでいる:true/掴んでいない:false
	 */
	public void setHolding(boolean holding) {
		this.holding = holding;
	}
	/**
	 * 掴んでいるかどうか確認する
	 * @return 掴んでいる:true/掴んでいない:false
	 */
	public boolean isHolding() {
		return holding;
	}
	/**
	 * 音声を停止する
	 */
	public void stop() {
		player.stop();
	}
	/**
	 * 音声を解放する
	 */
	public void release() {
		player.release();
	}
	/**
	 * 対象のオブジェクトとの距離が変更されたときに呼ばれる。
	 * @param target 対象のオブジェクト
	 */
	public void notifyDistanceChanged(SoundballCharacter target) {
		// 基本ボリューム（簡易的）
		int distance = getDistance(target);
		float vol = 0.5f * (float)(diagonal/2 - distance)/(float)(diagonal/2);
		
		// 左右のボリューム調整（簡易的）
		float balance = (float)(rect.centerX() - target.rect.centerX())/(float)viewRect.centerX();
		float left  = balance <= 0.0f ? vol * (1.0f - balance*-1.0f) : vol;
		float right = balance > 0.0f  ? vol * (1.0f - balance)       : vol;
		player.setVolume(left, right);
	}
}
