package net.grandnature.android.soundball.sound;

import java.util.Random;

/**
 * 音階に準じて音の波形を生成、管理するクラス。
 */
public class Note {
	private static int nextIndex = 0; // 次の音のindex
	private static double region = 0.5; // 振幅の幅 -0.5 から 0.5
	private byte[] soundData;
	private int sampleRate, noteIndex;
	private static final Random random = new Random();
	// Aマイナーペンタトニックスケールの5音
	public static final float[] NOTES = {
		220.0f, // A3
		264.0f, // C4
		293.3f, // D4
		330.0f, // E4
		396.0f, // G4
		440.0f, // A4
	};
	public Note(int sampleRate) {
		this.sampleRate = sampleRate;
		this.noteIndex = nextIndex++;
		if(nextIndex >= NOTES.length)
			nextIndex = 0;
		this.soundData = createSoundData();
	}
	public byte[] getSoundData() {
		return soundData;
	}
	public int getSampleRate() {
		return sampleRate;
	}
	private byte[] createSoundData() {
		byte[] b = null;

		// 生成する波形をランダムに選ぶ
		int type = random.nextInt(3);
		switch (type) {
		case 0:
			b = createSawtoothWave();
			break;
		case 1:
			b = createSquareWave();
			break;
		case 2:
		default:
			b = createSineWave();
			break;
		}
		return b;
	}
	// 正弦波を生成する
	private byte[] createSineWave() {
		byte[] b = new byte[sampleRate];
		for (int i = 0; i < sampleRate; i++) {
			double d = region * Math.sin(2.0 * Math.PI * NOTES[noteIndex] * i /sampleRate);
			b[i] = (byte)(d * 127.0);
		}
		return b;
	}
	// 矩形波を生成する
	private byte[] createSquareWave() {
		byte[] b = new byte[sampleRate];
		// その1. 符号を判断して生成
		for (int i = 0; i < sampleRate; i++) {
			double d = Math.sin(2.0 * Math.PI * NOTES[noteIndex] * i /sampleRate);
			b[i] = d > 0.0 ? (byte)(region * 127) : (byte)(region * -128);
		}
		// その2. 倍音の合成による生成
		// 1 or 10 or 15
//		int count = (Utils.getInt(3) + 1) * 5;
//		for (int i = 0; i < sampleRate; i++) {
//			double d = 0.0;
//			for (int j = 1; j <= count; j++) {
//			  if (j%2 == 1) 
//				d = d + region / j * Math.sin(2.0 * Math.PI * NOTES[noteIndex] * i * j /sampleRate);
//			}
//			b[i] = (byte)(d * 127.0);
//		}		
		return b;
	}
	// のこぎり波を生成する
	private byte[] createSawtoothWave() {
		byte[] b = new byte[sampleRate];
		// その1.　線形関数による生成
		for (int i = 0; i < sampleRate; i++) {
			double t = 1.0/sampleRate*i;
			double a = 1.0/NOTES[noteIndex];
			double d = region * 2 * (t/a - Math.floor(t/a + 0.5));
			b[i] = (byte)(d * 127.0);
		}
		// その2. 倍音の合成による生成
//		// 1 or 10 or 15
//		int count = (Utils.getInt(3) + 1)*5;
//		for (int i = 0; i < sampleRate; i++) {
//			double d = 0.0;
//			for (int j = 1; j <= count; j++) {
//				d = d + region / j * Math.sin(2.0 * Math.PI* NOTES[noteIndex] * i * j / sampleRate);	
//			}
//			b[i] = (byte)(d * 127.0);
//		}		
		return b;
	}
}


