package net.grandnature.android.soundball.sound;

import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;

/**
 * AudioTrackを使って音声を再生するクラス。
 */
public class SoundballPlayer {
	private AudioTrack at;
	/**
	 * 解放する。
	 */
	public void release() {
		try {
			if(at.getState() == AudioTrack.PLAYSTATE_PLAYING)
				at.stop();
			at.release();
		} catch (RuntimeException e) {
			e.printStackTrace();
		} finally {
			at = null;
		}
	}
	/**
	 * 再生する
	 * @param note 再生する音
	 */
	public void play(Note note) {
		if(at!=null) 
			at.reloadStaticData();
		else 
			createAudioTrack(note);
		at.play();
	}
	/**
	 * 停止する。
	 */
	public void stop() {
		if(at!=null && at.getPlayState() == AudioTrack.PLAYSTATE_PLAYING) 
			at.pause();
	}
	/**
	 * 音量を設定する
	 * @param left 左の音量
	 * @param right 右の音量
	 */
	public void setVolume(float left, float right) {
		if(at.getPlayState() == AudioTrack.PLAYSTATE_PLAYING) {
			at.setStereoVolume(left, right);
		}
	}
	// AudioTrackのインスタンスを生成する。
	private void createAudioTrack(Note note) {
		byte[] dat = note.getSoundData();
		int datLen = dat.length;
		// 1秒分のバッファを確保する		
		int buffersize = AudioTrack.getMinBufferSize(note.getSampleRate(), 
        		AudioFormat.CHANNEL_CONFIGURATION_MONO, AudioFormat.ENCODING_PCM_8BIT);
        if(buffersize <= datLen)
        	buffersize = datLen;
        // 8bitモノラル固定で初期化   
        at = new AudioTrack(
        		AudioManager.STREAM_MUSIC, note.getSampleRate(), AudioFormat.CHANNEL_CONFIGURATION_MONO, 
        		AudioFormat.ENCODING_PCM_8BIT, buffersize, AudioTrack.MODE_STATIC);
        at.setLoopPoints(0, buffersize, -1);
		int ret = at.write(dat, 0, dat.length);
		// 本来は、エラーに応じて適切に処理する	
		if(ret == AudioTrack.ERROR_INVALID_OPERATION){
			// エラー処理	
			}else if(ret == AudioTrack.ERROR_BAD_VALUE){
			// エラー処理	
			}
	}
}
